<?php

namespace Drupal\power_reviews\Controller;

use Drupal\node\Entity\Node;

/**
 * Controller routines for power_reviews routes.
 */
class PowerReviewsController {

  /**
   * Returns content for Write A Review page.
   *
   * @return array
   *   A render array representing the content.
   */
  public function writeReview() {
    $review_id = \Drupal::config('power_reviews.settings')->get('power_reviews_write_review');
    $power_reviews_type_of_implementation = \Drupal::config('power_reviews.settings')->get('power_reviews_type_of_implementation');
    $node_types = \Drupal::config('power_reviews.settings')->get('power_reviews_applied_content_types');

    $nids = [];
    $product_details = [];

    foreach ($node_types as $key => $value) {
      if ($value !== 0) {
        $nids = array_merge($nids, \Drupal::entityQuery('node')->condition('type', $key)->execute());
      }
    }

    $nodes = Node::loadMultiple($nids);

    if (\Drupal::config('power_reviews.settings')->get('power_reviews_feedless_implementation') == 'yes') {
      foreach ($nodes as $node) {
        if (!$node->get('field_power_reviews_page_id')->isEmpty()) {
          $product_array = [];
          $product_array = [
            'page_id' => $node->field_power_reviews_page_id->value,
            'name' => (NULL !== $node->getTitle()) ? $node->getTitle() : '',
            'sub_title' => !$node->get('field_product_sub_title')->isEmpty() ? $node->field_product_sub_title->value : '',
            'image_url' => !$node->get('field_power_reviews_image_url')->isEmpty() ? file_create_url($node->get('field_power_reviews_image_url')->entity->getFileUri()) : '',
            'color' => !$node->get('field_product_color')->isEmpty() ? $node->get('field_product_color')->getValue()[0]['color'] : '',
          ];
          $product_details[$node->nid->value] = $product_array;
        }
      }
    }

    if ($power_reviews_type_of_implementation == 'page_path') {
      $powerreviews_page_id = \Drupal::config('power_reviews.settings')->get('power_reviews_page_id');
    }
    else {
      $powerreviews_page_id = \Drupal::request()->query->get('pr_page_id');
    }

    $powerreview_api_details = power_reviews_initiate();
    $powerreviews_review_details = [
      'powerreviews_page_id' => $powerreviews_page_id,
      'power_reviews_write_review' => $review_id,
      'power_reviews_write_review_custom_header' => \Drupal::config('power_reviews.settings')->get('power_reviews_write_review_custom_header'),
      'product_details' => $product_details,
    ];

    if (\Drupal::config('power_reviews.settings')->get('power_reviews_write_review_custom_header') == "yes") {
      $review_rtn_array = [
        'power_reviews' => [
          'review_id' => $review_id,
          'header_title' => \Drupal::config('power_reviews.settings')->get('power_reviews_write_review_custom_header_title'),
          'header_body' => \Drupal::config('power_reviews.settings')->get('power_reviews_write_review_custom_header_body'),
        ],
        'product_details' => $product_details,
      ];

      $build = [
        '#theme' => 'write_a_review',
        '#data' => $review_rtn_array,
      ];

      $build['#attached']['library'][] = 'power_reviews/power-reviews-js';
      $build['#attached']['library'][] = 'power_reviews/power-reviews-war-script';
      $build['#attached']['drupalSettings']['powerreview_api_details'] = $powerreview_api_details;
      $build['#attached']['drupalSettings']['powerreviews_review_details'] = $powerreviews_review_details;

      return $build;
    }
    else {
      return "<div id='" . $review_id . "'></div>";
    }
  }

}
